import { Component, Input, OnInit } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material';
import { Observable } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { BaseComponent } from '../../../../core/common/base/base.component';
import { DataTableRequest } from '../../../../core/model/data-table/data-table-request';
import { DatasetProfileModel } from '../../../../core/model/dataset/dataset-profile';
import { DmpProfileDefinition } from '../../../../core/model/dmp-profile/dmp-profile';
import { ExternalSourceItemModel } from '../../../../core/model/external-sources/external-source-item';
import { DatasetProfileCriteria } from '../../../../core/query/dataset-profile/dataset-profile-criteria';
import { DmpProfileCriteria } from '../../../../core/query/dmp/dmp-profile-criteria';
import { RequestItem } from '../../../../core/query/request-item';
import { DmpProfileService } from '../../../../core/services/dmp/dmp-profile.service';
import { DmpService } from '../../../../core/services/dmp/dmp.service';
import { ExternalSourcesService } from '../../../../core/services/external-sources/external-sources.service';
import { MultipleAutoCompleteConfiguration } from '../../../../library/auto-complete/multiple/multiple-auto-complete-configuration';
import { SingleAutoCompleteConfiguration } from '../../../../library/auto-complete/single/single-auto-complete-configuration';
import { LanguageResolverService } from '../../../../services/language-resolver/language-resolver.service';
import { AddResearcherComponent } from '../add-researcher/add-researcher.component';
import { AvailableProfilesComponent } from '../available-profiles/available-profiles.component';
import { startWith } from 'rxjs/internal/operators';

@Component({
	selector: 'app-general-tab',
	templateUrl: './general-tab.component.html',
	styleUrls: ['./general-tab.component.scss']
})
export class GeneralTabComponent extends BaseComponent implements OnInit {

	@Input() formGroup: FormGroup = null;

	filteringOrganisationsAsync = false;
	filteringResearchersAsync = false;
	filteredProfilesAsync = false;
	filteredOrganisations: ExternalSourceItemModel[];
	filteredResearchers: ExternalSourceItemModel[];
	filteredProfiles: DatasetProfileModel[];

	profilesAutoCompleteConfiguration: MultipleAutoCompleteConfiguration;
	organisationsAutoCompleteConfiguration: MultipleAutoCompleteConfiguration;
	researchersAutoCompleteConfiguration: MultipleAutoCompleteConfiguration;
	dmpProfileAutoCompleteConfiguration: SingleAutoCompleteConfiguration;

	selectedDmpProfileDefinition: DmpProfileDefinition;

	constructor(
		private dmpProfileService: DmpProfileService,
		private externalSourcesService: ExternalSourcesService,
		private _service: DmpService,
		private dialog: MatDialog,
		public languageResolverService: LanguageResolverService,
	) {
		super();
	}

	ngOnInit() {

		if (this.formGroup.get('definition')) { this.selectedDmpProfileDefinition = this.formGroup.get('definition').value; }
		this.registerFormEventsForDmpProfile();

		this.dmpProfileAutoCompleteConfiguration = {
			filterFn: this.dmpProfileSearch.bind(this),
			initialItems: (extraData) => this.dmpProfileSearch(''),
			displayFn: (item) => item['label'],
			titleFn: (item) => item['label']
		};

		this.profilesAutoCompleteConfiguration = {
			filterFn: this.filterProfiles.bind(this),
			initialItems: (excludedItems: any[]) => this.filterProfiles('').map(result => result.filter(resultItem => excludedItems.map(x => x.id).indexOf(resultItem.id) === -1)),
			displayFn: (item) => item['label'],
			titleFn: (item) => item['label'],
			subtitleFn: (item) => item['description']
		};

		this.organisationsAutoCompleteConfiguration = {
			filterFn: this.filterOrganisations.bind(this),
			initialItems: (excludedItems: any[]) => this.filterOrganisations('').map(result => result.filter(resultItem => excludedItems.map(x => x.id).indexOf(resultItem.id) === -1)),
			displayFn: (item) => item['name'],
			titleFn: (item) => item['name'],
			subtitleFn: (item) => item ? item['tag'] : null
		};

		this.researchersAutoCompleteConfiguration = {
			filterFn: this.filterResearchers.bind(this),
			initialItems: (excludedItems: any[]) => this.filterResearchers('').map(result => result.filter(resultItem => excludedItems.map(x => x.id).indexOf(resultItem.id) === -1)),
			displayFn: (item) => item['name'],
			titleFn: (item) => item['name'],
			subtitleFn: (item) => item ? item['tag'] : null
		};
	}

	registerFormEventsForDmpProfile(definitionProperties?: DmpProfileDefinition): void {
		this.formGroup.get('profile').valueChanges
			.pipe(
			takeUntil(this._destroyed))
			.subscribe(Option => {
				if (Option instanceof Object) {
					this.selectedDmpProfileDefinition = null;
					this.dmpProfileService.getSingle(Option.id)
						.pipe(takeUntil(this._destroyed))
						.subscribe(result => {
							this.selectedDmpProfileDefinition = result.definition;
						});
				} else {
					this.selectedDmpProfileDefinition = null;
				}
				this.selectedDmpProfileDefinition = definitionProperties;
			})
	}

	dmpProfileSearch(query: string) {
		let fields: Array<string> = new Array();
		var request = new DataTableRequest<DmpProfileCriteria>(0, 10, { fields: fields });
		request.criteria = new DmpProfileCriteria();
		return this.dmpProfileService.getPaged(request).map(x => x.data);
	}

	// onCallbackSuccess(): void {
	// 	this.uiNotificationService.snackBarNotification(this.isNew ? this.language.instant('GENERAL.SNACK-BAR.SUCCESSFUL-CREATION') : this.language.instant('GENERAL.SNACK-BAR.SUCCESSFUL-UPDATE'), SnackBarNotificationLevel.Success);
	// 	this.router.navigate(['/plans']);
	// }

	// onCallbackError(error: any) {
	// 	this.setErrorModel(error.error);

	// 	//this.validateAllFormFields(this.formGroup);
	// }

	// public setErrorModel(validationErrorModel: ValidationErrorModel) {
	// 	Object.keys(validationErrorModel).forEach(item => {
	// 		(<any>this.dmp.validationErrorModel)[item] = (<any>validationErrorModel)[item];
	// 	});
	// }

	filterOrganisations(value: string): Observable<ExternalSourceItemModel[]> {

		this.filteredOrganisations = undefined;
		this.filteringOrganisationsAsync = true;

		return this.externalSourcesService.searchDMPOrganizations(value);
	}

	filterResearchers(value: string): Observable<ExternalSourceItemModel[]> {

		this.filteredResearchers = undefined;
		this.filteringResearchersAsync = true;

		return this.externalSourcesService.searchDMPResearchers({ criteria: { name: value, like: null } });
	}

	filterProfiles(value: string): Observable<DatasetProfileModel[]> {

		this.filteredProfiles = undefined;
		this.filteredProfilesAsync = true;

		const request = new RequestItem<DatasetProfileCriteria>();
		const criteria = new DatasetProfileCriteria();
		criteria.like = value;
		request.criteria = criteria;
		return this._service.searchDMPProfiles(request);
	}

	addResearcher() {
		const dialogRef = this.dialog.open(AddResearcherComponent, {
			data: this.formGroup.get('researchers')
		});
		dialogRef.afterClosed().pipe(takeUntil(this._destroyed)).subscribe(result => {
			if (result) {
				const fullName = result.firstName + " " + result.lastName;
				this.formGroup.get('researchers').value.push({
					label: null,
					name: fullName,
					id: "dmp:" + fullName,
					status: 0,
					tag: null,
				});
			}
		});
	}

	availableProfiles() {
		const dialogRef = this.dialog.open(AvailableProfilesComponent, {
			data: {
				profiles: this.formGroup.get('profiles')
			}
		});

		return false;
	}
}
